/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */
package cz.xelfi.quoridor;

/**
 * Represents position and number of available fences of a player on
 * a {@link Board}.
 */
public final class Player extends Object {

    final int x;
    final int y;
    /** number of fences this player has */
    final int f;
    /** the direction of players end line */
    final Direction endDirection;

    Player(int x, int y, int f, Player.Direction endDir) {
        super();
        this.x = x;
        this.y = y;
        this.f = f;
        this.endDirection = endDir;
    }

    /** Returns the x-coordinate of the player.
     * @return  number from 0 to 16
     */
    int getXInternal() {
        return x;
    }

    /** Returns the y-coordinate of the player.
     * @return  number from 0 to 16
     */
    int getYInternal() {
        return y;
    }

    /** Returns position of the player from 0-8.
     *
     * @return integer from 0-8
     */
    public int getColumn() {
        return x / 2;
    }

    /** The y-position of the player.
     * @return integer from 0-8
     */
    public int getRow() {
        return y / 2;
    }

    /** How much fences is still available for the player.
     * 
     * @return number of fences this player still has
     */
    public int getFences() {
        return f;
    }

    @Override
    public String toString() {
        return "Player[" + getColumn() + "," + getRow() + "," + f + "," + endDirection + "]";
    }

    @Override
    public int hashCode() {
        return 8 * x + 2 * y + 4 * f + 7 + endDirection.hashCode();
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof Player) {
            Player p = (Player) o;
            return x == p.x && y == p.y && f == p.f && endDirection.equals(p.endDirection);
        }
        return false;
    }

    /** Possible directions of player's moves.
     */
    public enum Direction {

        NORTH('P', 16), WEST((char) -1, -1), EAST((char) -1, -1), SOUTH('Q', 0);
        final char player;
        final int finalLine;

        private Direction(char ch, int fl) {
            this.player = ch;
            this.finalLine = fl;
        }

        final boolean reached(Player p) {
            return p.getYInternal() == finalLine;
        }

        static Direction valueOf(char ch) throws IllegalPositionException {
            for (Direction direction : values()) {
                if (direction.name().charAt(0) == ch) {
                    return direction;
                }
            }
            throw new IllegalPositionException("No direction " + ch);
        }
    }
}
